﻿Type=Class
Version=5.9
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private ws As WebSocket 'ignore
	' will hold our page information
	Public page As ABMPage
	' page theme
	Private theme As ABMTheme
	' to access the constants
	Private ABM As ABMaterial 'ignore	
	' name of the page, must be the same as the class name (case sensitive!)
	Public Name As String = "AboutPage"
	' name of the app, same as in ABMApplication
'	Public AppName As String = "template"
	
	Private ABMPageId As String = ""
	' your own variables
	Private myToastId As Int
	
	Private gen As ABMGenerator  ' required to build the CRUD code.
	Private genname As String = "report.txt"

	
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	' build the local structure IMPORTANT!
	BuildPage
End Sub

Private Sub WebSocket_Connected (WebSocket1 As WebSocket)
	'----------------------MODIFICATION-------------------------------
	Log("Connected")
		
	ws = WebSocket1
	
	ABMPageId = ABM.GetPageID(page, Name,ws)
	
	Dim session As HttpSession = ABM.GetSession(ws, ABMShared.SessionMaxInactiveIntervalSeconds)
	If session.IsNew Then
		session.Invalidate
		ABMShared.NavigateToPage(ws, "", "./")
		Return
	End If
	
	'  Set needs auth to true
	ABMShared.NeedsAuthorization = True
	
	If ABMShared.NeedsAuthorization Then
		If session.GetAttribute2("IsAuthorized", "") = "" Then
			ABMShared.NavigateToPage(ws, ABMPageId, "../")
			Return
		End If
	End If
	
	ABM.UpdateFromCache(Me, ABMShared.CachedPages, ABMPageId, ws)
	If page.ComesFromPageCache Then
		' when we have a page that is cached it doesn't matter if it comes or not from a new connection we serve the cached version.
		Log("Comes from cache")
		page.Refresh
		page.FinishedLoading
	Else
		If page.WebsocketReconnected Then
			Log("Websocket reconnected")
			' when we have a client that doesn't have the page in cache and it's websocket reconnected and also it's session is new - basically when the client had internet problems and it's session (and also cache) expired before he reconnected so the user has content in the browser but we don't have any on the server. So we need to reload the page.
			' when a client that doesn't have the page in cache and it's websocket reconnected but it's session is not new - when the client had internet problems and when he reconnected it's session was valid but he had no cache for this page we need to reload the page as the user browser has content, reconnected but we have no content in cache
			ABMShared.NavigateToPage (ws, ABMPageId, "./" & page.PageHTMLName)
		Else
			' when the client did not reconnected it doesn't matter if the session was new or not because this is the websockets first connection so no dynamic content in the browser ... we are going to serve the dynamic content...
			Log("Websocket first connection")
			page.Prepare
			ConnectPage
		End If
	End If
	Log("  -- This Page ID: "&ABMPageId)
End Sub

Private Sub WebSocket_Disconnected
	Log("Disconnected")
End Sub

Sub Page_ParseEvent(Params As Map)
	Dim eventName As String = Params.Get("eventname")
	Dim eventParams() As String = Regex.Split(",",Params.Get("eventparams"))
	If eventName = "beforeunload" Then
		Log("preparing for url refresh")
		ABM.RemoveMeFromCache(ABMShared.CachedPages, ABMPageId)
		Return
	End If
	
	'  Very Helpful to log control clicks and discover what params are required - and such !!!!
	Log(" *** Page Event name: "&eventName&"  "&Params) ' this is used to see what component was clicked and what the parameters are...
	
	
	Dim caller As Object = page.GetEventHandler(Me, eventName)
	If caller = Me Then
		If SubExists(Me, eventName) Then
			Params.Remove("eventname")
			Params.Remove("eventparams")
			' BEGIN NEW DRAGDROP
			If eventName = "page_dropped" Then
				page.ProcessDroppedEvent(Params)
			End If
			' END NEW DRAGDROP
			Select Case Params.Size
				Case 0
					CallSub(Me, eventName)
				Case 1
					CallSub2(Me, eventName, Params.Get(eventParams(0)))
				Case 2
					If Params.get(eventParams(0)) = "abmistable" Then
						Dim PassedTables As List = ABM.ProcessTablesFromTargetName(Params.get(eventParams(1)))
						CallSub2(Me, eventName, PassedTables)
					Else
						CallSub3(Me, eventName, Params.Get(eventParams(0)), Params.Get(eventParams(1)))
					End If
				Case Else
					' cannot be called directly, to many param
					CallSub2(Me, eventName, Params)
			End Select
		End If
	Else
		CallSubDelayed2(caller, "ParseEvent", Params) 'ignore
	End If
End Sub

public Sub BuildTheme()
	
	' start with the base theme defined in ABMShared Code Module
	' Private theme As ABMTheme   -  in class_globals above...

	theme.Initialize("pagetheme")
	theme.AddABMTheme(ABMShared.MyTheme)

End Sub




public Sub BuildPage()
	' initialize the theme
	BuildTheme
	page.InitializeWithTheme(Name, "/ws/" & ABMShared.AppName & "/" & Name, False, ABMShared.SessionMaxInactiveIntervalSeconds, theme)
	
	' show the spinning cicles while page is loading....
	page.ShowLoader=True
	page.PageHTMLName = "index.html"
	page.PageTitle = ""  ' You can also set this as a property in "ABMShared.BuildNavigationBar" below...
	
	'  Google SEO stuff...
	page.PageDescription = ""
	page.PageKeywords = ""
	page.PageSiteMapPriority = ""
	page.PageSiteMapFrequency = ABM.SITEMAP_FREQ_YEARLY
		
	' faint green dot (on title bar) when connected - red when not connected with web socket
	page.ShowConnectedIndicator = True
	
	ABMShared.BuildNavigationBar(page, "My {B} About Page {/B} ", "../images/logo.png",  "",  "",  "")
	
	page.AddRowsM( 5, True, 0, 10, "").AddCellsOSMP( 1, 0, 0, 0, 12, 12, 12, 0,  0, 0, 0, "")
	page.AddRowsM( 1, True, 0, 10, "").AddCellsOSMP( 2, 0, 0, 0, 6, 6, 6, 0,  0, 0, 0, "")
	page.AddRowsM( 3, True, 0, 10, "").AddCellsOSMP( 1, 0, 0, 0, 12, 12, 12, 0,  0, 0, 0, "")
	

	'IMPORTANT - Build the Grid before you start adding components ( with ConnectPage()!!! )
	page.BuildGrid
	
End Sub

Sub ConnectPage()
	' See the ABMPageTemplate for the tutorial!
	
	ABMShared.ConnectNavigationBar(page)
	' add header
	page.Cell(1,1).AddComponent(ABMShared.BuildLabel(page,  "hdr1",  "About / Help (Members Only) Landing Page", ABM.SIZE_H3, "",0,""))
	page.Cell(3,1).AddComponent(ABMShared.BuildLabel(page,  "hdr2",  "Lesson 5 - {B}ABMGenerator{/B} "&CRLF&CRLF&" Don't Like Hand Writing Code For Your Modal Sheets???  Well, you are not alone here..."&CRLF&CRLF&"{B} Let ABMGenerator Write It For You!{/B} "   , ABM.SIZE_H5, "",0,""))
	page.Cell(4,1).AddComponent(ABMShared.BuildParagraphBQWithZDepth(page,   "hdr3",   " Clean, error free code.  Check out the {B}'BuildCrudCode'{/B} method in this About Page"&CRLF&CRLF& "Click the 'Build' button below to generate the code, then the 'Show' button to load results in a ABMCodeLabel..."))
	' This button is centered in cell 2 by using a theme (see -	MyTheme.AddCellTheme("cnter") in ABMShared module


	Dim btnpids As ABMButton   ' what's in a name???  - It's the ID that COUNTS!!!  ("geNcrud")
	btnpids.InitializeRaised( page ,  "gencrud", "mdi-action-lock-outline", ABM.ICONALIGN_LEFT,"Build My CRUD Code","amber")
	page.Cell(6,1).UseTheme("cnter")
	page.Cell(6,1).AddComponent(btnpids)


	Dim btnpids As ABMButton  ' what's in a name???  - It's the ID that COUNTS!!!  ("geTcrud")
	btnpids.InitializeRaised( page ,  "getcrud", "mdi-action-assignment", ABM.ICONALIGN_LEFT,"Show My Built Code","amber")
	page.Cell(6,2).UseTheme("cnter")
	page.Cell(6,2).AddComponent(btnpids)


	
	page.Refresh ' IMPORTANT
 
	' NEW, because we use ShowLoaderType=ABM.LOADER_TYPE_MANUAL
	page.FinishedLoading 'IMPORTANT
	
End Sub


' event to capture MSGBOX2 activities...
Sub page_MsgboxResult(returnName As String, result As String)

	If result = ABM.MSGBOX_RESULT_OK Then   '  the ok button was clicked...
		Log(" OK was pressed   Return name "&returnName&"  result: "&result)
	End If

	Select Case returnName
		Case "dogen"     ' this is from the gencrud_clicked method...  
			Select Case result
			
				Case  "abmok" '  ABM.INPUTBOX_RESULT_CANCEL, ABM.INPUTBOX_RESULT_CLOSE, ABM.INPUTBOX_RESULT_ESC, ABM.INPUTBOX_RESULT_OVERLAY, ABM.INPUTBOX_RESULT_INVALID
					Log(" Calling BuildCrudCode ")
					BuildCrudCode   ' run this method on YES...
					
			End Select
	End Select
     
End Sub


Sub getcrud_clicked(target As String) ' show the source generated
	
	'page.Msgbox2("dogen", "",  "Generate CRUD Code Now?","YES","Cancel", False, ABM.MSGBOX_TYPE_QUESTION,  False,   ABM.MSGBOX_POS_CENTER_CENTER,   "")
	
	If File.Exists(File.DirApp,genname) Then
 	    Dim repfile As String
		repfile = File.ReadString(File.DirApp,genname)
		Dim code2 As StringBuilder
		code2.Initialize
		code2.Append(repfile).Append(CRLF)
		page.Cell(7,1).AddComponent( ABMShared.BuildLabel( page, "lb11", "The report.txt file", ABM.SIZE_H4, "", 0, "30px"))
		page.Cell(7,1).AddComponent(BuildCodeBlock(page, "code2", code2))
		page.Cell(7,1).Refresh
		
'		Dim code3 As String = $" &repfile& "$
'		page.Cell(6,1).AddComponent(ABMShared.BuildCodeBlockFromSmartString(page, "code3", code3))
	   
	Else
		page.Msgbox("ret","","File: "&genname&" does not exist!","OK",False,ABM.MSGBOX_POS_BOTTOM_CENTER,"")
	End If
	

	If File.Exists(File.DirApp,"DBM.txt") Then   ' get the sql support methods too!!!
		page.Cell(8,1).AddComponent(ABMShared.BuildLabel(page,"lb21","The DBM.txt file",ABM.SIZE_H4,"",0,"30px"))
		
		Dim repfile As String
		repfile = File.ReadString(File.DirApp,"DBM.txt")
		Dim code2 As StringBuilder
		code2.Initialize
		code2.Append(repfile).Append(CRLF)
		page.Cell(8,1).AddComponent(BuildCodeBlock(page, "code3", code2))
		page.Cell(8,1).Refresh
		
'		Dim code3 As String = $" &repfile& "$
'		page.Cell(6,1).AddComponent(ABMShared.BuildCodeBlockFromSmartString(page, "code3", code3))
	   
	Else
		page.Msgbox("ret","","File: "&genname&" does not exist!","OK",False,ABM.MSGBOX_POS_BOTTOM_CENTER,"")
	End If

	
End Sub



' event for generate button...
Sub gencrud_clicked(target As String)
	
	page.Msgbox2("dogen", "",  "Generate CRUD Code Now?","YES","Cancel", False, ABM.MSGBOX_TYPE_QUESTION,  False,   ABM.MSGBOX_POS_CENTER_CENTER,   "")
	
End Sub

Sub BuildCrudCode

	Dim MyCrudDef As ABMGeneratorCRUDDefinition
	
	
	' uncomment the line below to check the properties and methods available with ABMGeneratorCRUDDefinition
	' Just a few items are used in our example.  I don't claim to know what all of these mean - or do...
	' MyCrudDef.
	
	
	Log(" Running BuildCrudCode ")
	
	
	
	' This is the structure we shall define that the code generator will use to build our sheets and supporting methods
	' 98 percent of the sheet code is written for you. There are places we shall add addtional code to make the sheet complete - to our specifc requirements
	' Some Setxx lines are optional and others required.  Each will be defined.
	' Each section must have the same number of properties set.  It is a good idea to space each cell, for each row, equally to ensure this rule is met.
	' the next 3 lines show a sample set - nicely formatted and spaced...
'	MyCrudDef.Set04ComponentTypes(   Array As Int    (ABM.GEN_NONE     , ABM.GEN_TEXT     , ABM.GEN_TEXT    , ABM.GEN_SWITCH   , ABM.GEN_SWITCH        , ABM.GEN_SWITCH    , ABM.GEN_SWITCH     , ABM.GEN_SWITCH  , ABM.GEN_CHECKBOX    ,  ABM.GEN_INTEGER  , ABM.GEN_TEXT ))
'	MyCrudDef.Set05DefaultValues(    Array As String (""               , " "              , " "             , "true"           , "false"               , "false"           , "false"            , "false"         , "true"              , "93"              , ""           ))
'	MyCrudDef.Set06ComboQueries(     Array As String (""               , ""               , ""              , ""               , cmbZones              , ""                , ""                 , ""              , ""                  , ""                , ""           ))
	' leave the above lines commented... (for visual explanation only)...
	
	
	
	' ( Not Required )
	' This could be used to populate a combo item list for our sheet....  
	' This example uses a conditional Select from a table - however - it is NOT used in this lesson - just defined for informational purposes.
	'
	Dim cmbZones As String = "SELECT pk, name from picklists where grp = 1"



	' (Required)
	' these are the field names from our MySQL table that we want to show on our modal sheet.  
	' *************************  below is an example of generated code *****************************
	'   Dim ABMGenreportp2 As ABMSwitch     -   "ABMGen" (component prefix)   "report" (the uniqueName of our crud sheet)   "p2" (the field name)
	'  ABMGenreportp2.Initialize(page, "ABMGenreportp2", "Show", "Hide", False, "")  - init the Switch component...
	'  ABMGenreportModal.Content.CellR(1,1).AddComponent(ABMGenreportp2)             - add component to a row / cell
	'  ABMGenreportp2.State = False                                                  - the default state of our switch
	' *************************  above is an example of generated code *****************************
	'
	MyCrudDef.Set01FieldNames(       Array As String ("pk"             , "repname"        , "repfile"       , "p1"             , "p2"                  , "p3"              , "p4"               , "p5"            , "active"            ,  "comp_id"        ,"rtype"       ))
		
		
		
	' (Required)
	' This section defines where rows and cells will be placed.  
	' "" implies it will be hidden.  
	' "1:0:4"  1 means place on row 1.  0 means offest on S,M,L devices.   4 means make this cell 4 cols wide.
	' all cell widths for each row must up to 12...
	' you notice this same structure is repeated for row 2 ("2:0:4"  , "2:0:4"   , "2:0:4") / row 2, 0 offset, 4 cols wide each...
	' same for row 3 - but row 4 is one cell - 12 wide....
	'
	MyCrudDef.Set02RowOffsetSizes(   Array As String ( ""              , "1:0:4"          , "1:0:4"         , "1:0:4"          , "2:0:4"               , "2:0:4"           , "2:0:4"            , "3:0:4"         , "3:0:4"             , "3:0:4"           , "4:0:12"     ))
	
	
	
	
	' these are our label titles for each cell component.  This is commented because we are using it as a reference (place holder). Why??? - See next section where this definition is actually used.
	' We will use these text values to ADD a title to each switch component.  The generator was created BEFORE switch titles were introduced into ABMaterial....
'	MyCrudDef.Set03LabelTexts(       Array As String (""               , "Report Name"    , "File Name"     , "Show Dates "    , "Show Driver/Vehicle" , "Show Contractor" , "Show Summary"     , "Show Search"   ,  "Report Active"    ,  "Company"        ,"Edit"        ))
	'
	'
	' (Required)
	' these are our label titles for each cell component.  However, when we use a switch, the text represents what the ON / OFF text will show as a label.
	'
	' after the final code has been generated and added to our page, we will modify some sections to COMPLETE the resulting sheet...
	MyCrudDef.Set03LabelTexts(       Array As String (""               , "Report Name"    , "File Name"     , "Show,Hide"      , "Show,Hide"           , "Show,Hide"       , "Show,Hide"        , "Show,Hide"     ,  "Show,Hide"        ,  "Company"        ,"Edit"        ))
	
	
	
	
	' (Required)
	' this section defines the TYPE of ABM component that will be placed into each cell.  Currently, there are 15 possible choices.  Try ABM.Gen...  to see your options.
	' ABM.gen
	' 	
	MyCrudDef.Set04ComponentTypes(   Array As Int    (ABM.GEN_NONE     , ABM.GEN_TEXT     , ABM.GEN_TEXT    , ABM.GEN_SWITCH   , ABM.GEN_SWITCH        , ABM.GEN_SWITCH    , ABM.GEN_SWITCH     , ABM.GEN_SWITCH  , ABM.GEN_CHECKBOX    ,  ABM.GEN_INTEGER  , ABM.GEN_TEXT ))
	
	

	
	' (Required)
	' this section defines our "default values" when ADDING (Inserting) a new item using the modal sheet.  When "editing", the values are sourced from the selected record.
	' notice boolean type components (Switch in this case) use "true" or "false"
	'
	MyCrudDef.Set05DefaultValues(    Array As String (""               , " "              , " "             , "true"           , "false"               , "false"           , "false"            , "false"         , "true"              , "93"              , ""           ))
	
	
	
	' (Not Required)
	'  this section will build methods used to populate combos for each ABMCombo component used - if desired. 
	' uses the statement - 	( Dim cmbZones As String = "SELECT pk, name from picklists where grp = 1" ) - shown above...
	'  
	MyCrudDef.Set06ComboQueries(     Array As String (""               , ""               , ""              , ""               , cmbZones              , ""                , ""                 , ""              , ""                  , ""                , ""           ))




	' (Not Required)
	' this section defines validation methods that will be injected into each component when saving (insert or update)
	' NotMin1 and NotEmpty are the names of the method generated and used...  Shown are the examples...
	' 
	'	Sub ABMGenNotMin1(value As Boolean) As Boolean
	'		' TODO by the programmer!   - you determine what to say or do in this event....
	'		Return True
	'	End Sub
	'
	'	' method to check if a certain field is valid - Not Empty!!!!
	'	Sub ABMGenNotEmpty(value As String) As Boolean
	'		' TODO by the programmer!
	'		Return True   - returns true as NOT to fail validation.....
	'	End Sub
	'
	MyCrudDef.Set08ValidationMethods(Array As String (""               , ""               , ""              , ""               , "NotMin1"             , "NotMin1"         , "NotMin1"          , ""              , "NotMin1"           , ""                , "NotEmpty"   ))




	' there may be certain fields that are NOT used in the query update process.  In this example, "pk" is our primary key and an AutoIncrement field in MySQL table, so don't touch it!!!!
	'
	MyCrudDef.Set10UseInUpdates(     Array As Boolean(False            , True             , True            , True             , True                  , True              , True               , True            , True                , False             , True         ))
	
	
	
	
		
	' OK, with everything defined, generate the output code please!	
	'	
	' Generate the text files based on configuration above....
	'
	' NOTE: a DBM.txt file is also generated for your SQL methods....  
	'
	' params (where to put generated file, file and sheet name, definition (MyCrudDef), default sheet size) 
	gen.GenerateCRUDSheet(File.DirApp, "report",   MyCrudDef,  ABM.MODALSHEET_SIZE_LARGE)
	genname = "report.txt"  ' ignore... used below for example...
	
	Dim str As String = File.GetUri(File.DirApp,genname)
	page.Msgbox("msg"," This file may be found at: "&CRLF &str ,"File: '"&genname&"' Was Created!"&CRLF&"Use Show Button to Load in ABMCodeLabel"  ,"OK",False,ABM.MSGBOX_POS_BOTTOM_CENTER,"")
	
	
	
'	' *************************  A template for each of the above ***************************** *************************  A template for each of the above *****************************
'	MyCrudDef.Set01FieldNames(       Array As String ("pk"             , "repname"        , "repfile"       , "p1"             , "p2"                  , "p3"              , "p4"               , "p5"            , "active"            ,  "comp_id"        ,"rtype"       ))
'	MyCrudDef.Set02RowOffsetSizes(   Array As String ( ""              , "1:0:4"          , "1:0:4"         , "1:0:4"          , "2:0:4"               , "2:0:4"           , "2:0:4"            , "3:0:4"         , "3:0:4"             , "3:0:4"           , "4:0:12"     ))
''	' not used... MyCrudDef.Set03LabelTexts(       Array As String (""               , "Report Name"    , "File Name"     , "Show Dates "    , "Show Driver/Vehicle" , "Show Contractor" , "Show Summary"     , "Show Search"   ,  "Report Active"    ,  "Company"        ,"Edit"        ))
'	MyCrudDef.Set03LabelTexts(       Array As String (""               , "Report Name"    , "File Name"     , "Show,Hide"      , "Show,Hide"           , "Show,Hide"       , "Show,Hide"        , "Show,Hide"     ,  "Show,Hide"        ,  "Company"        ,"Edit"        ))
'	MyCrudDef.Set04ComponentTypes(   Array As Int    (ABM.GEN_NONE     , ABM.GEN_TEXT     , ABM.GEN_TEXT    , ABM.GEN_SWITCH   , ABM.GEN_SWITCH        , ABM.GEN_SWITCH    , ABM.GEN_SWITCH     , ABM.GEN_SWITCH  , ABM.GEN_CHECKBOX    ,  ABM.GEN_INTEGER  , ABM.GEN_TEXT ))
'	MyCrudDef.Set05DefaultValues(    Array As String (""               , " "              , " "             , "true"           , "false"               , "false"           , "false"            , "false"         , "true"              , "93"              , ""           ))
'	MyCrudDef.Set06ComboQueries(     Array As String (""               , ""               , ""              , ""               , cmbZones              , ""                , ""                 , ""              , ""                  , ""                , ""           ))
'	MyCrudDef.Set08ValidationMethods(Array As String (""               , ""               , ""              , ""               , "NotMin1"             , "NotMin1"         , "NotMin1"          , ""              , "NotMin1"           , ""                , "NotEmpty"   ))
'	MyCrudDef.Set10UseInUpdates(     Array As Boolean(False            , True             , True            , True             , True                  , True              , True               , True            , True                , False             , True         ))
'	' *************************  A template for each of the above ***************************** *************************  A template for each of the above *****************************
	
	
End Sub


Sub Page_NavigationbarClicked(Action As String, Value As String)
	' saving the navigation bar position
	page.SaveNavigationBarPosition
	If Action = "LogOff" Then
		ABMShared.LogOff(page)
		Return
	End If

	ABMShared.NavigateToPage(ws, ABMPageId, Value)
End Sub


 ' method stolen from @alwaysbusy....
Sub BuildCodeBlock(page1 As ABMPage, id As String, code As StringBuilder) As ABMCodeLabel
	Dim codelab As ABMCodeLabel
	codelab.Initialize(page1, id, code.ToString, "vb")
	Return codelab
End Sub


